/*
 *   ksame 0.4 - simple Game
 *   Copyright (C) 1997,1998  Marcus Kreutzberger
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *    
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>

#include <qpainter.h> 
#include <qpixmap.h> 

#include <kapp.h>
#include <kconfig.h>
#include <kiconloader.h>

#include <time.h>
#include <kglobal.h>
#include <kstddirs.h>

#include "StoneWidget.moc"

StoneWidget::StoneWidget( QWidget *parent, int x, int y ) 
     : QWidget(parent,"StoneWidget"), stonefield(x,y) {

     stonemap = QPixmap(locate("appdata", "stones.png"));
     debug("stones.png %ix%i",stonemap.width(),stonemap.height());

     ASSERT(!stonemap.isNull());
     
     slice=0;
     maxslices=30;
     maxcolors=4;
     
     sizex=x;
     sizey=y;
     debug("size: %i %i",sizex,sizey);
     
     stone_width=stonemap.width()/(maxslices+1);
     stone_height=stonemap.height()/maxcolors;
     
     field_height=stone_height*sizey;
     field_width=stone_width*sizex;
     
     setMouseTracking(true);
     
     QColor c(115,115,115);
     setBackgroundColor(c);
     
     // emit s_sizechanged();
     startTimer( 100 ); 
     history.setAutoDelete(true); 

}

StoneWidget::~StoneWidget() {  
     setMouseTracking(false);
     killTimers();
     //   debug("~StoneWidget\n");
}

unsigned int
StoneWidget::board() {
     return stonefield.getBoard();
}

int
StoneWidget::score() {
     return stonefield.getScore();
}

QSize
StoneWidget::size() {
     return QSize(sizex,sizey);
}

int
StoneWidget::colors() {
     return stonefield.getColors();
}

QSize 
StoneWidget::sizeHint () {
     return QSize(field_width,field_height);
}

QSize
StoneWidget::minimumSizeHint() {
     return QSize(field_width,field_height);
}

void 
StoneWidget::newGame(unsigned int board,int colors) {
     //     debug("StoneWidget::newgame board %i colors %i",board,colors);
     stonefield.newGame(board,colors);
     history.clear();
     emit s_newgame();
     emit s_colors(stonefield.getColors());
     emit s_board(stonefield.getBoard());
}

void 
StoneWidget::reset() {
     stonefield.reset();
     history.clear();
     emit s_newgame();
}

void 
StoneWidget::unmark() {
     stonefield.unmark();
     emit s_marked(0);
}

int
StoneWidget::undo(int count) {
     if (stonefield.isGameover()) return 0;
     
     int ret_val=stonefield.undo(count);

     QPoint p=mapFromGlobal(cursor().pos());
     int x=p.x();
     int y=p.y();
     if (x<0||y<0||x>=field_width||y>=field_height) {
	  emit s_score(stonefield.getMarked()); 
	  return ret_val;
     }
     
     int marked=stonefield.mark(x/stone_width,y/stone_height);
     emit s_marked(marked);
     slice=0;
     emit s_score(stonefield.getScore()); 
     return ret_val;
}

bool 
StoneWidget::isGameover() {
     return stonefield.isGameover();
}

void
StoneWidget::readProperties(KConfig *conf) {
     ASSERT(conf);
     
     history.clear();
     
     if (!conf->hasKey("Board")||
	 !conf->hasKey("Colors")||
	 !conf->hasKey("Stones")) {
	  return;
     }
     debug("readprop");
     newGame(conf->readNumEntry("Board"),conf->readNumEntry("Colors"));

     QStrList list;
     conf->readListEntry("Stones",list);
     
     for (const char *item=list.first();item;item=list.next()) {
	  int x=-1,y=-1;
	  if (sscanf(item,"%02X%02X",&x,&y)!=2) break;
	  history.append(new QPoint(x,y));
	  stonefield.remove(x,y);
     }
}


void
StoneWidget::saveProperties(KConfig *conf) {
     ASSERT(conf);
     
     QStrList list(true);
     QString tmp;
     
     for (QPoint *item=history.first();item;item=history.next()) {
	  tmp.sprintf("%02X%02X",item->x(),item->y());
	  list.append(tmp.ascii());
     }

     conf->writeEntry("Stones",list);
     conf->writeEntry("Board",stonefield.getBoard());
     conf->writeEntry("Colors",stonefield.getColors());
}

void
StoneWidget::timerEvent( QTimerEvent * ) {
     QPoint p=mapFromGlobal(cursor().pos());
     int x=p.x();
     int y=p.y();
     if (x<0||y<0||x>=field_width||y>=field_height)
	  stonefield.unmark();
     slice=(slice+1)%maxslices;
     paintEvent(0);
}

void
StoneWidget::paintEvent( QPaintEvent *e ) {
     int x,y;
     int redraw;
  
     //    debug("slice:%i\n",slice);
     Stone *stone=stonefield.getField();
     
     for (y=0;y<field_height;y+=stone_height) {
	  for (x=0;x<field_width;x+=stone_width) {
	       // debug("stone %i %i",x,y);
	       redraw=stone->marked||stone->changed;
	       
	       if (!redraw&&e) {
		    QRect r(x,y,stone_width,stone_height);
		    redraw=r.intersects(e->rect());
	       }
	       if (redraw) {
		    stone->changed=false;
		    if (stone->color) {
			 bitBlt(this,x,y,
				&stonemap,
				stone->marked?stone_width*slice:0,
				(stone->color-1)*stone_height,
				stone_width,stone_height,CopyROP,FALSE);
			 
		    } else {
			 bitBlt(this,x,y,
				&stonemap,stone_width*maxslices,0,
				stone_width,stone_height,CopyROP,FALSE);
		    }
	       }
	       stone++;  // naechster Stein.
	  }
     }
}

void
StoneWidget::mousePressEvent ( QMouseEvent *e) {

     if (stonefield.isGameover()) return;
     
     int x=e->pos().x();
     int y=e->pos().y();
     if (x<0||y<0||x>=field_width||y>=field_height) return;
     
     int sx=x/stone_width;
     int sy=y/stone_height;

     if (stonefield.remove(sx, sy)) {
	  history.append(new QPoint(sx, sy));
	  
	  emit s_remove(sx, sy); 

	  stonefield.mark(sx,sy);
	  emit s_marked(stonefield.getMarked());
	  
	  emit s_score(stonefield.getScore()); 
	  if (stonefield.isGameover()) emit s_gameover(); 
     }
}

void
StoneWidget::mouseMoveEvent ( QMouseEvent *e) { 

     if (stonefield.isGameover()) {
	  stonefield.unmark();
	  emit s_marked(0);
	  return; 
     }

     int x=e->pos().x();
     int y=e->pos().y();
     if (x<0||y<0||x>=field_width||y>=field_height) return;

     int marked=stonefield.mark(x/stone_width,y/stone_height);
     if (marked>=0) {
	  emit s_marked(marked);
	  slice=0;
     }
}








